#!/usr/bin/python2

# Input is beta diversity distance matrix for the mock samples.
# Sample names must be:
#	bok ex koz.v34 koz.v4 koz.v45 v13fa v13fb v31ra v31rb v35fa v53ra v53rb 
#	v69fa v69fb v96ra v96rb

# Usage:
#	make_beta_dev_histograms.py otutable_filename > output.txt

import sys

FileName = sys.argv[1]
f = open(FileName)
HdrLine = f.readline()
HdrFields = HdrLine[:-1].split('\t')
Samples = HdrFields[1:]
SampleCount = len(Samples)

N=20
BinSize = 1.0/N

All_Bins = []
KozSameSample_Bins = []
SameV_Bins = []
DiffV_Bins = []
Ex_Bins = []

for i in range(0, N):
	All_Bins.append(0)
	KozSameSample_Bins.append(0)
	SameV_Bins.append(0)
	DiffV_Bins.append(0)
	Ex_Bins.append(0)

def IsSameV(Name1, Name2):
	V1 = GetV(Name1)
	V2 = GetV(Name2)
	return V1 == V2

def IsKozSameSample(Name1, Name2):
	if not (Name1.startswith("koz") and Name1.startswith("koz")):
		return False
	return IsSameV(Name1, Name2)

def GetV(SampleName):
	if SampleName.find("bok") >= 0:
		return "v4"
	if SampleName.startswith("koz"):
		Fields = SampleName.split(".")
		v = Fields[1]
		assert v.startswith(v)
		return v
	if SampleName.find("ex") >= 0:
		return ""
	if SampleName.startswith("v"):
		v = SampleName[0:3]
		assert len(v) == 3
		return v
	assert False

for i in range(0, SampleCount):
	Line = f.readline()
	Fields = Line[:-1].split('\t')
	assert len(Fields) == SampleCount + 1
	SampleNamei = Samples[i]
	Samplei_IsEx = (SampleNamei.find("ex") >= 0)
	for j in range(0, SampleCount):
		SampleNamej = Samples[j]
		Samplej_IsEx = (SampleNamei.find("ex") >= 0)
		Value = float(Fields[j+1])
		assert Value >= 0.0 and Value <= 1.0
		if Value == 1.0:
			Bin = N - 1
		else:
			Bin = int(Value*N)
		assert Bin <= N

		if Samplei_IsEx or Samplei_IsEx:
			if Samplei_IsEx and Samplei_IsEx:
				Ex_Bins[Bin] += 1
			continue

		All_Bins[Bin] += 1
		if IsKozSameSample(SampleNamei, SampleNamej):
			KozSameSample_Bins[Bin] += 1
		if IsSameV(SampleNamei, SampleNamej):
			SameV_Bins[Bin] += 1
		else:
			DiffV_Bins[Bin] += 1

def PrintBins(Name, Bins):
	print ""
	print Name
	Tot = 0
	for Bin in range(0, N):
		n = Bins[Bin]
		Tot += n
	for Bin in range(0, N):
		Lo = Bin*BinSize
		Hi = (Bin + 1)*BinSize
		n = Bins[Bin]
                if Tot == 0:
                    assert n == 0
                    f = 0.0
                else:
                    f = float(n)/Tot
		print "%.4f-%.4f\t%u\t%.5f" % (Lo, Hi, n, f)

PrintBins("All", All_Bins)
PrintBins("KozSameSample", KozSameSample_Bins)
PrintBins("SameV", SameV_Bins)
PrintBins("DiffV", DiffV_Bins)
PrintBins("Ex", Ex_Bins)
