#!/usr/bin/python2

import sys

# Calculate probability that a pair of tags is assigned to the
# same OTU. Input is a tabbed text file formatted as below giving
# the closed-reference OTU assignment for each tag for all GG sequences.
# Label is the GG sequence label from which the tag was extracted.
# The "# " at the start of each line is to make a python comment,
# they are not part of the format.
# "@" means the tag does not exist for the primer pair (because
# the sequence is trunctated before v1 / after v9, or because
# there are too many primer mismatches).
# A precomputed table can be downloaded from:
#	http://drive5.com/downloads/ggv_tab.txt.gz

# Label   v13     v34     v35     v4      v45     v69
# 287144  352503  280459  280459  541859  541859  @
# 4361279 @       137445  548434  1033018 849156  @

f = open(sys.argv[1])

HdrLine = f.readline()
HdrFields = HdrLine.strip().split('\t')
assert HdrFields[0] == "Label"

Vs = HdrFields[1:]

VCount = len(Vs)
Pairs = []
for i in range(0, VCount):
	Vi = Vs[i]
	for j in range(i+1, VCount):
		Vj = Vs[i]
		Pair = Vi + "\t" + Vj
		Pairs.append(Pair)

PairCount = len(Pairs)
PairToSameCount = []
PairToSameCountA = []
for i in range(0, PairCount):
	PairToSameCount.append(0)
	PairToSameCountA.append(0)

AllSameCount = 0
AllPresentCount = 0
N = 0
while 1:
	Line = f.readline()
	if len(Line) == 0:
		break
	N += 1
	Fields = Line[:-1].split('\t')
	OTUs = Fields[1:]
	assert len(OTUs) == VCount
	PairIndex = 0
	for i in range(0, VCount):
		OTUi = OTUs[i]
		for j in range(0, i):
			OTUj = OTUs[j]
			if OTUi == OTUj and OTUi != "@" and OTUi != "*":
				PairToSameCount[PairIndex] += 1
			PairIndex += 1

	AllSame = True
	AllPresent = True
	if OTUs[0] == "@" or OTUs[0] == "*":
		AllSame = False
	for i in range(0, VCount):
		OTU = OTUs[i]
		if OTU != OTUs[0]:
			AllSame = False
		if OTU == "@":
			AllPresent = False
	if AllSame:
		AllSameCount += 1
	if AllPresent:
		AllPresentCount += 1

	if AllPresent:
		PairIndex = 0
		for i in range(0, VCount):
			OTUi = OTUs[i]
			for j in range(0, i):
				OTUj = OTUs[j]
				if OTUi == OTUj and OTUi != "@" and OTUi != "*":
					PairToSameCountA[PairIndex] += 1
				PairIndex += 1

def GetPct(x, y):
	return (x*100.0)/y

PairIndex = 0
for i in range(1, VCount):
	Vi = Vs[i]
	s = Vi
	for j in range(0, i):
		n = PairToSameCount[PairIndex]
		PairIndex += 1
		Pct = (n*100.0)/N
		s += "\t%.1f%%" % (Pct)
	print s
s = ""
for i in range(0, VCount):
	V = Vs[i]
	s += "\t" + V
print s

print ""
PairIndex = 0
for i in range(1, VCount):
	Vi = Vs[i]
	s = Vi
	for j in range(0, i):
		n = PairToSameCountA[PairIndex]
		PairIndex += 1
		Pct = (n*100.0)/AllPresentCount
		s += "\t%.1f%%" % (Pct)
	print s
s = ""
for i in range(0, VCount):
	V = Vs[i]
	s += "\t" + V
print s

print ""
print "Present   %10u (%.1f%%)" % (AllPresentCount, GetPct(AllPresentCount, N))
print "All same  %10u (%.1f%%)" % (AllSameCount, GetPct(AllSameCount, AllPresentCount))
